//19 May 2024
//IdeaZero Designs 
//Spatial clock v1.0

#include <RTClib.h>
#include <FastLED.h>

RTC_DS3231 rtc;

#define DATA_PIN     4
#define NUM_LEDS    12
#define BRIGHTNESS  230
#define LED_TYPE    WS2812B
#define COLOR_ORDER GRB

struct CRGB leds[NUM_LEDS];

#define FRAMES_PER_SECOND  120

DateTime old; // Variable to compare new and old time, to see if it has moved on.

int timehours = 0;
int timeminutes = 0;
int timeseconds = 0;

int previousHour = -1;
bool PM = false;


unsigned long previousMillis = 0; // Variable to store the last time LED was updated
const long interval_ = 1000; // Interval at which to blink (in milliseconds)

void setup () {
  delay(3000);
  Serial.begin(9600);

  FastLED.addLeds<LED_TYPE,DATA_PIN,COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
  FastLED.setBrightness(BRIGHTNESS);

  // SETUP RTC MODULE
  if (! rtc.begin()) {
    Serial.println("RTC module is NOT found");
    Serial.flush();
    while (1);
  }

  //Init code
  sinelon();
  FastLED.show(); 
  
  // automatically sets the RTC to the date & time on PC this sketch was compiled
  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  // manually sets the RTC with an explicit date & time, for example to set
  // January 21, 2021 at 3am you would call:
   //rtc.adjust(DateTime(2024, 5, 19, 0, 22, 0));
}

void loop () {
  DateTime now = rtc.now();
//  Serial.print("ESP32 RTC Date Time: ");
//  Serial.print(now.year(), DEC);
//  Serial.print('/');
//  Serial.print(now.month(), DEC);
//  Serial.print('/');
//  Serial.print(now.day(), DEC);
//  Serial.print(" (");
//  Serial.print(daysOfWeek[now.dayOfTheWeek()]);
//  Serial.print(") ");
//  Serial.print(now.hour(), DEC);
//  Serial.print(':');
//  Serial.print(now.minute(), DEC);
//  Serial.print(':');
//  Serial.println(now.second(), DEC);
  
  timehours = now.hour();
  timeminutes = now.minute();
  timeseconds = now.second();

  PM = false;
  if (now.hour() >= 12) {
    PM = true;}
  
  clearLEDs();
  
  timehours = (timehours%12);
  if (timehours == 0) timehours = 12;
  //Serial.print(timehours);Serial.print(" : ");Serial.print(timeminutes);Serial.print(" : ");Serial.println(timeseconds);
 
  int currentHour = now.hour();
  if (currentHour != previousHour) 
  {
  previousHour = currentHour;
  sinelon();
  }
   
  minimalClock();
  FastLED.show(); 
  FastLED.delay(1000/FRAMES_PER_SECOND); 
  //delay(1000); // delay 1 seconds
  
}

void minimalClock()
{
  if (PM == false) // For AM = White
  {
   leds[timehours]=CRGB(255, 255, 255); 
  }
  if (PM == true) // For PM = Orange
  {
    leds[timehours]=CRGB(238, 165, 0); 
  }
  
  leds[timeminutes/5]=CRGB(0, 230, 60); // Green for minutes (12 LEDs, 5 minutes per LED)

  if ((timeseconds%2) == 0){
    leds[timeminutes/5]=CRGB(0, 230, 60);
  }
  else {
    leds[timeminutes/5]=CRGB(0, 0, 0);
  } 
}


void clearLEDs()
{      
  for (int i = 0; i < NUM_LEDS; i++) // Set all the LEDs to off
    {
      leds[i].r = 0;
      leds[i].g = 0;
      leds[i].b = 0;
    }
}

void sinelon()
{
unsigned long startTime = millis();
while (millis() - startTime < 3500) {
  // a colored dot sweeping back and forth, with fading trails
  fadeToBlackBy( leds, NUM_LEDS, 20);
  int pos = beatsin16( 13, 0, NUM_LEDS-1 );
  leds[pos] += CHSV( 128, 255, 255);
  FastLED.show(); 
}
}
